@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -467,12 +467,16 @@ function install_setup_get_templates() {
 	@ini_set('zlib.output_compression', '0');
 
 	$templates = array(
+		'Cacti_Stats.xml.gz',
+		'Cisco_ASA_Appliance.xml.gz',
 		'Cisco_Router.xml.gz',
+		'Citrix_NetScaler_VPX.xml.gz',
 		'Generic_SNMP_Device.xml.gz',
 		'Local_Linux_Machine.xml.gz',
 		'NetSNMP_Device.xml.gz',
-		'Windows_Device.xml.gz',
-		'Cacti_Stats.xml.gz'
+		'PING_Advanced_Ping.xml.gz',
+		'Synology_NAS.xml.gz',
+		'Windows_Device.xml.gz'
 	);
 
 	$path = $config['base_path'] . '/install/templates';
@@ -870,34 +874,37 @@ function import_colors() {
 	return true;
 }
 
-function log_install_debug($section, $string) {
-	log_install_and_file(POLLER_VERBOSITY_DEBUG, $string, $section);
+function log_install_debug($section, $text, $background = false) {
+	log_install_and_file(POLLER_VERBOSITY_DEBUG, $text, $section, $background);
 }
 
-function log_install_low($section, $string) {
-	log_install_and_file(POLLER_VERBOSITY_LOW, $string, $section);
+function log_install_low($section, $text, $background = false) {
+	log_install_and_file(POLLER_VERBOSITY_LOW, $text, $section, $background);
 }
 
-function log_install_medium($section, $string) {
-	log_install_and_file(POLLER_VERBOSITY_MEDIUM, $string, $section);
+function log_install_medium($section, $text, $background = false) {
+	log_install_and_file(POLLER_VERBOSITY_MEDIUM, $text, $section, $background);
 }
 
-function log_install_high($section, $string) {
-	log_install_and_file(POLLER_VERBOSITY_HIGH, $string, $section);
+function log_install_high($section, $text, $background = false) {
+	log_install_and_file(POLLER_VERBOSITY_HIGH, $text, $section, $background);
 }
 
-function log_install_always($section, $string) {
-	log_install_and_file(POLLER_VERBOSITY_NONE, $string, $section);
+function log_install_always($section, $text, $background = false) {
+	log_install_and_file(POLLER_VERBOSITY_NONE, $text, $section, $background);
 }
 
-function log_install_and_file($level, $string, $section = '') {
+function log_install_and_file($level, $text, $section = '', $background = false) {
 	$level = log_install_level_sanitize($level);
 	$name = 'INSTALL:';
 	if (!empty($section)) {
 		$name = 'INSTALL-' . strtoupper($section) . ':';
 	}
-	cacti_log(log_install_level_name($level) . ': ' . $string, false, $name, $level);
-	log_install_to_file($section, $string, FILE_APPEND, $level);
+	cacti_log(log_install_level_name($level) . ': ' . $text, false, $name, $level);
+	log_install_to_file($section, $text, FILE_APPEND, $level);
+	if ($background) {
+		set_config_option('install_updated', microtime(true));
+	}
 }
 
 function log_install_section_level($section) {
@@ -1073,14 +1080,18 @@ function install_full_sync() {
 		WHERE id > 1
 		AND disabled = ""');
 
+	log_install_always('sync', 'Found ' . cacti_sizeof($pollers) . ' poller(s) to sync');
 	if (cacti_sizeof($pollers)) {
 		foreach($pollers as $poller) {
+			log_install_debug('sync', 'Poller ' . $poller['id'] . ' has a status of ' . $poller['status'] . ' with gap ' . $poller['gap']);
 			if (($poller['status'] == POLLER_STATUS_NEW) ||
 				($poller['status'] == POLLER_STATUS_DOWN) ||
 				($poller['status'] == POLLER_STATUS_DISABLED)) {
 				$skipped[] = $poller['id'];
-			} elseif ($gap < $gap_time) {
+			} elseif ($poller['gap'] < $gap_time) {
+				log_install_medium('sync', 'Replicating to Poller ' . $poller['id']);
 				if (replicate_out($poller['id'])) {
+					log_install_debug('sync', 'Completed replication to Poller ' . $poller['id']);
 					$success[] = $poller['id'];
 
 					db_execute_prepared('UPDATE poller
@@ -1088,6 +1099,7 @@ function install_full_sync() {
 						WHERE id = ?',
 						array($id));
 				} else {
+					log_install_debug('sync', 'Failed replication to Poller ' . $poller['id']);
 					$failed[] = $poller['id'];
 				}
 			} else {
@@ -1095,6 +1107,7 @@ function install_full_sync() {
 			}
 		}
 	}
+	log_install_debug('sync', 'Success: ' . cacti_sizeof($success) . ', Failed: ' . cacti_sizeof($failed) . ', Skipped: ' . cacti_sizeof($skipped) . ', Total: ' . cacti_sizeof($pollers));
 
 	return array(
 		'success' => $success,
@@ -1104,4 +1117,3 @@ function install_full_sync() {
 		'total'   => cacti_sizeof($pollers)
 	);
 }
-
