@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -1249,14 +1249,27 @@ function boost_poller_bottom() {
 
 		boost_update_snmp_statistics();
 
+		$boost_log     = read_config_option('path_boost_log');
+		$boost_logdir  = dirname($boost_log);
+
+		if ($boost_log != '') {
+			if (!is_writable($boost_log) || !is_dir($boost_logdir) || !is_writable($boost_logdir)) {
+				boost_debug("WARNING: Boost log '$boost_log' does not exist or is not writable!");
+
+				cacti_log("WARNING: Boost log '$boost_log' is not writable!", false, 'BOOST');
+
+				$boost_log = '';
+			}
+		}
+
 		$command_string = read_config_option('path_php_binary');
-		if (read_config_option('path_boost_log') != '') {
+		if ($boost_log != '') {
 			if ($config['cacti_server_os'] == 'unix') {
-				$extra_args    = '-q ' . $config['base_path'] . '/poller_boost.php --debug';
-				$redirect_args =  '>> ' . read_config_option('path_boost_log') . ' 2>&1';
+				$extra_args    = '-q '  . $config['base_path'] . '/poller_boost.php --debug';
+				$redirect_args =  '>> ' . $boost_log . ' 2>&1';
 			} else {
 				$extra_args    = '-q ' . $config['base_path'] . '/poller_boost.php --debug';
-				$redirect_args = '>> ' . read_config_option('path_boost_log');
+				$redirect_args = '>> ' . $boost_log;
 			}
 		} else {
 			$extra_args = '-q ' . $config['base_path'] . '/poller_boost.php';
