@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -22,9 +22,13 @@
  +-------------------------------------------------------------------------+
 */
 
-/* exec_poll - executes a command and returns its output
-   @arg $command - the command to execute
-   @returns - the output of $command after execution */
+/**
+ * exec_poll - executes a command and returns its output
+ *
+ * @param  (string) $command - the command to execute
+ *
+ * @return (string) the output of $command after execution
+ */
 function exec_poll($command) {
 	global $config;
 
@@ -51,15 +55,18 @@ function exec_poll($command) {
 	return $output;
 }
 
-/* exec_poll_php - sends a command to the php script server and returns the
-     output
-   @arg $command - the command to send to the php script server
-   @arg $using_proc_function - whether or not this version of php is making use
-     of the proc_open() and proc_close() functions (php 4.3+)
-   @arg $pipes - the array of r/w pipes returned from proc_open()
-   @arg $proc_fd - the file descriptor returned from proc_open()
-   @returns - the output of $command after execution against the php script
-     server */
+/**
+ * exec_poll_php - sends a command to the php script server and returns the output
+ *
+ * @param  (string)   $command - the command to send to the php script server
+ * @param  (bool)     $using_proc_function - whether or not this version of php is making use
+ *                    of the proc_open() and proc_close() functions (php 4.3+)
+ * @param  (array)    $pipes - the array of r/w pipes returned from proc_open()
+ * @param  (resource) $proc_fd - the file descriptor returned from proc_open()
+ *
+ * @return (string) - the output of $command after execution against the php script
+ *   server
+ */
 function exec_poll_php($command, $using_proc_function, $pipes, $proc_fd) {
 	global $config;
 
@@ -112,11 +119,16 @@ function exec_poll_php($command, $using_proc_function, $pipes, $proc_fd) {
 	return $output;
 }
 
-/* exec_background - executes a program in the background so that php can continue
-     to execute code in the foreground
-   @arg $filename - the full pathname to the script to execute
-   @arg $args - any additional arguments that must be passed onto the executable
-   @arg $redirect_args - any additional arguments for file re-direction.  Otherwise output goes to /dev/null */
+/**
+ * exec_background - executes a program in the background so that php can continue
+ *   to execute code in the foreground.
+ *
+ * @param  (string) $filename      - the full pathname to the script to execute
+ * @param  (string) $args          - any additional arguments that must be passed onto the executable
+ * @param  (string) $redirect_args - any additional arguments for file re-direction.  Otherwise output goes to /dev/null
+ *
+ * @return (void)
+ */
 function exec_background($filename, $args = '', $redirect_args = '') {
 	global $config, $debug;
 
@@ -155,9 +167,14 @@ function file_escaped($file) {
 	return false;
 }
 
-/* file_exists_2gb - fail safe version of the file exists function to correct
-     for errors in certain versions of php.
-   @arg $filename - the name of the file to be tested. */
+/**
+ * file_exists_2gb - fail safe version of the file exists function to correct
+ *   for errors in certain versions of php.
+ *
+ * @param  (string) $filename - the name of the file to be tested.
+ *
+ * @return (int) 1 if the file exists otherwise 0
+ */
 function file_exists_2gb($filename) {
 	global $config;
 
@@ -170,10 +187,15 @@ function file_exists_2gb($filename) {
 	}
 }
 
-/* update_reindex_cache - builds a cache that is used by the poller to determine if the
-     indexes for a particular data query/host have changed
-   @arg $host_id - the id of the host to which the data query belongs
-   @arg $data_query_id - the id of the data query to rebuild the reindex cache for */
+/**
+ * update_reindex_cache - builds a cache that is used by the poller to determine if the
+ *   indexes for a particular data query/host have changed
+ *
+ * @param  (int) $host_id - the id of the host to which the data query belongs
+ * @param  (int) $data_query_id - the id of the data query to rebuild the reindex cache for
+ *
+ * @return (void)
+ */
 function update_reindex_cache($host_id, $data_query_id) {
 	global $config;
 
@@ -394,15 +416,20 @@ function poller_update_poller_reindex_from_buffer($host_id, $data_query_id, &$re
 	poller_push_reindex_only_data_to_main($host_id, $data_query_id);
 }
 
-/* process_poller_output - grabs data from the 'poller_output' table and feeds the *completed*
-     results to RRDtool for processing
-  @arg $rrdtool_pipe - the array of pipes containing the file descriptor for rrdtool
-  @arg $remainder - don't use LIMIT if true */
-function process_poller_output(&$rrdtool_pipe, $remainder = false) {
+/**
+ * process_poller_output - grabs data from the 'poller_output' table and feeds the *completed*
+ *   results to RRDtool for processing
+ *
+ * @param  (resource) $rrdtool_pipe - the array of pipes containing the file descriptor for rrdtool
+ * @param  (int)      $remainder - don't use LIMIT if true
+ *
+ * @return (int) - The number of rrdfiles processed
+ */
+function process_poller_output(&$rrdtool_pipe, $remainder = 0) {
 	global $config, $debug;
 
-	static $have_deleted_rows = true;
 	static $rrd_field_names = array();
+	static $checked_bad     = false;
 
 	include_once($config['library_path'] . '/rrd.php');
 
@@ -410,21 +437,13 @@ function process_poller_output(&$rrdtool_pipe, $remainder = false) {
 	$rrds_processed = 0;
 	$max_rows = 40000;
 
-	if ($remainder) {
-		/* check if too many rows pending */
-		$rows = db_fetch_cell('SELECT COUNT(local_data_id)
-			FROM poller_output');
-
-		if ($rows > $max_rows && $have_deleted_rows === true) {
-			$limit = ' LIMIT ' . $max_rows;
-		} else {
-			$limit = '';
-		}
-	} else {
-		$limit = 'LIMIT ' . $max_rows;
+	if ($remainder == 0) {
+		$remainder = $max_rows;
 	}
 
-	$have_deleted_rows = false;
+	cacti_log("Processing Poller Output with $remainder maximum items to be processed", false, 'POLLER', POLLER_VERBOSITY_HIGH);
+
+	$limit = 'LIMIT ' . $max_rows;
 
 	/* create/update the rrd files */
 	$results = db_fetch_assoc("SELECT po.output, po.time,
@@ -545,7 +564,6 @@ function process_poller_output(&$rrdtool_pipe, $remainder = false) {
 					$k++;
 					if ($k % 10000 == 0) {
 						db_execute('DELETE FROM poller_output WHERE local_data_id IN (' . implode(',', $data_ids) . ')');
-						$have_deleted_rows = true;
 						$data_ids = array();
 						$k = 0;
 					}
@@ -557,7 +575,6 @@ function process_poller_output(&$rrdtool_pipe, $remainder = false) {
 
 		if ($k > 0) {
 			db_execute('DELETE FROM poller_output WHERE local_data_id IN (' . implode(',', $data_ids) . ')');
-			$have_deleted_rows = true;
 		}
 
 		/* process dsstats information */
@@ -578,19 +595,60 @@ function process_poller_output(&$rrdtool_pipe, $remainder = false) {
 			FROM poller_output');
 
 		/* to much records in poller_output, process in chunks */
-		if ($rows && $remainder) {
-			$rrds_processed += process_poller_output($rrdtool_pipe, $remainder);
+		if ($rows && $remainder == $max_rows) {
+			$rrds_processed += process_poller_output($rrdtool_pipe, $rows < $max_rows ? $rows : $max_rows);
+
+			$running = db_fetch_cell('SELECT COUNT(*)
+				FROM poller_time
+				WHERE end_time = "0000-00-00"');
+
+			if ($running == 0 && !$checked_bad) {
+				// Remove recently deleted items from the poller_output table
+				db_execute('DELETE FROM poller_output WHERE local_data_id NOT IN (SELECT id FROM data_local)');
+
+				// Itentify data sources that are somehow not aligned
+				$items = db_fetch_assoc('SELECT rrd_num,
+					COUNT(DISTINCT po.local_data_id, po.rrd_name) AS ids, dt.name, dl.host_id,
+					GROUP_CONCAT(DISTINCT po.local_data_id) AS local_data_ids
+					FROM poller_output AS po
+					LEFT JOIN poller_item AS pi
+					ON po.local_data_id = pi.local_data_id
+					LEFT JOIN data_local AS dl
+					ON po.local_data_id = dl.id
+					LEFT JOIN data_template AS dt
+					ON dl.data_template_id = dt.id
+					GROUP BY po.local_data_id
+					HAVING rrd_num IS NULL OR rrd_num != ids
+					ORDER BY dt.name');
+
+				if (cacti_sizeof($items)) {
+					cacti_log(sprintf('WARNING: There are %s Data Sources not returning all data leaving rows in the poller output table.  Details to follow.', cacti_sizeof($items)), false, 'POLLER');
+					$prevName = '';
+					foreach($items as $item) {
+						if ($prevName != $item['name']) {
+							cacti_log(sprintf('WARNING: Data Template \'%s\' is impacted by lack of complete information', $item['name']), false, 'POLLER');
+							$prevName = $item['name'];
+
+							db_execute('DELETE FROM poller_output WHERE local_data_id IN(' . $item['local_data_ids'] . ')');
+						}
+					}
+				}
+
+				$checked_bad = true;
+			}
 		}
 	}
 
 	return $rrds_processed;
 }
 
-/** update_resource_cache - place the cacti website in the poller_resource_cache
+/**
+ * update_resource_cache - place the cacti website in the poller_resource_cache
+ *   for remote pollers to consume
  *
- *  for remote pollers to consume
- * @param int $poller_id    - The id of the poller.  1 is the main system
- * @return null             - No data is returned
+ * @param  (int) $poller_id    - The id of the poller.  1 is the main system
+ *
+ * @return (void)
  */
 function update_resource_cache($poller_id = 1) {
 	global $config, $remote_db_cnn_id;
@@ -757,13 +815,15 @@ function update_resource_cache($poller_id = 1) {
 	}
 }
 
-/** cache_in_path - check to see if the directory in question has changed.
- *  If so, send its data into the resource cache table
+/**
+ * cache_in_path - check to see if the directory in question has changed.
+ *   If so, send its data into the resource cache table
  *
- * @param string $path      - The path to look for changes
- * @param string $type      - The patch types being cached
- * @param bool   $recursive - Should the path be scanned recursively
- * @return null             - No data is returned
+ * @param  (string) $path      - The path to look for changes
+ * @param  (string) $type      - The patch types being cached
+ * @param  (bool)   $recursive - Should the path be scanned recursively
+ *
+ * @return (void)
  */
 function cache_in_path($path, $type, $recursive = true) {
 	global $config;
@@ -801,7 +861,11 @@ function cache_in_path($path, $type, $recursive = true) {
 		if (basename($path) == 'config.php' && strpos($path, 'plugins') !== false) {
 			// Allow replication of plugin based config.php files
 			$exclude = false;
+		} elseif (basename($path) == 'config_local.php') {
+			// Don't cache a plugin config_local.php
+			$exclude = true;
 		} elseif (basename($path) == 'config.php') {
+			// Don't cache Cacti's config.php
 			$exclude = true;
 		} elseif (array_search(basename($path), $excluded_dirs_files, true) !== false) {
 			$exclude = true;
@@ -826,13 +890,15 @@ function cache_in_path($path, $type, $recursive = true) {
 	}
 }
 
-/** update_db_from_path - store the actual file in the databases resource cache.
- *  Skip the include/config.php if it exists
+/**
+ * update_db_from_path - store the actual file in the databases resource cache.
+ *   Skip the include/config.php if it exists
  *
- * @param string $path      - The path to look for changes
- * @param string $type      - The patch types being cached
- * @param bool   $recursive - Should the path be scanned recursively
- * @return null             - No data is returned
+ * @param  (string) $path      - The path to look for changes
+ * @param  (string) $type      - The patch types being cached
+ * @param  (bool)   $recursive - Should the path be scanned recursively
+ *
+ * @return (void)
  */
 function update_db_from_path($path, $type, $recursive = true) {
 	global $config;
@@ -845,11 +911,13 @@ function update_db_from_path($path, $type, $recursive = true) {
 		while (($entry = $pobject->read()) !== false) {
 			if (!should_ignore_from_replication($entry)) {
 				$spath = ltrim(trim(str_replace($config['base_path'], '', $path), '/ \\') . '/' . $entry, '/ \\');
+
 				if (is_dir($path . DIRECTORY_SEPARATOR . $entry)) {
 					if ($recursive) {
 						update_db_from_path($path . DIRECTORY_SEPARATOR . $entry, $type, $recursive);
 					}
-				} elseif (basename($path) == 'config.php' && strpos($path, 'plugins') === false) {
+				} elseif (basename($spath) == 'config.php' && strpos($path, 'plugins') === false) {
+					// Don't cache Cacti's config.php
 					continue;
 				} elseif (basename($path) == '.travis.yml') {
 					continue;
@@ -925,13 +993,15 @@ function update_db_from_path($path, $type, $recursive = true) {
 	}
 }
 
-/** resource_cache_out - push the cache from the cacti database to the
- *  remote database.  Check PHP files for errors
+/**
+ * resource_cache_out - push the cache from the cacti database to the
+ *   remote database.  Check PHP files for errors before placing
+ *   them on the remote pollers file system.
  *
- * before placing them on the remote pollers file system.
- * @param string $type      - The path type being cached
- * @param string $path      - The path to store the contents
- * @return null             - No data is returned
+ * @param (string) $type      - The path type being cached
+ * @param (string) $path      - The path to store the contents
+ *
+ * @return (void)
  */
 function resource_cache_out($type, $path) {
 	global $config, $remote_db_cnn_id;
@@ -1051,11 +1121,13 @@ function resource_cache_out($type, $path) {
 	}
 }
 
-/** md5sum_path - get a recursive md5sum on an entire directory.
+/**
+ * md5sum_path - get a recursive md5sum on an entire directory.
  *
- * @param string $path      - The path to check for the md5sum
- * @param bool   $recursive - The path should be verified recursively
- * @return null             - No data is returned
+ * @param  (string) $path      - The path to check for the md5sum
+ * @param  (bool)   $recursive - The path should be verified recursively
+ *
+ * @return (void)
  */
 function md5sum_path($path, $recursive = true) {
     if (!is_dir($path)) {
@@ -1098,6 +1170,15 @@ function md5sum_path($path, $recursive = true) {
     return md5(implode('', $filemd5s));
 }
 
+/**
+ * poller_puth_to_remote_db_connect - given the device or poller_id connect
+ *   to the data collector.
+ *
+ * @param  (int)    device_or_poller - the id of the object
+ * @param  (bool)   is_poller - don't let cacti guess, the id is a poller
+ *
+ * @return (bool|resource) The connection or false when the connection fails
+ */
 function poller_push_to_remote_db_connect($device_or_poller, $is_poller = false) {
 	global $config;
 	static $device_poller_ids = array();
@@ -1129,6 +1210,15 @@ function poller_push_to_remote_db_connect($device_or_poller, $is_poller = false)
 	return $rcnn_id;
 }
 
+/**
+ * poller_connect_to_remote - this function connects to the remote
+ *   data collector and returns either false or the connection
+ *   resource.
+ *
+ * @param  (int)    poller_id - the remote poller id
+ *
+ * @return (bool|resource) The connection or false when the connection fails
+ */
 function poller_connect_to_remote($poller_id) {
 	global $config, $local_db_cnn_id;
 
@@ -1183,12 +1273,25 @@ function poller_connect_to_remote($poller_id) {
 	return $rcnn_id;
 }
 
+/**
+ * replicate_out - this function sends table changes from the resource
+ *   cache to the remote database.  This happens as a result of a full
+ *   sync within Cacti.
+ *
+ * @param  (int)    remote_poller_id - the poller to send data to
+ * @param  (string) class - the class of data to push to the poller
+ *
+ * @return (bool)
+ */
 function replicate_out($remote_poller_id = 1, $class = 'all') {
 	global $config;
 
+	replicate_log('Attempting to replicate to Poller ' . $remote_poller_id);
+
 	$rcnn_id = poller_connect_to_remote($remote_poller_id);
 
 	if ($rcnn_id === false) {
+		replicate_log('Failed to connect to Poller ' . $remote_poller_id . ' Database');
 		return false;
 	}
 
@@ -1264,6 +1367,18 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 		replicate_out_table($rcnn_id, $data, 'version', $remote_poller_id);
 	}
 
+	// Plugin tables
+	if ($class == 'all' || $class == 'plugins') {
+		$data = db_fetch_assoc('SELECT * FROM plugin_config');
+		replicate_out_table($rcnn_id, $data, 'plugin_config', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM plugin_hooks');
+		replicate_out_table($rcnn_id, $data, 'plugin_hooks', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM plugin_realms');
+		replicate_out_table($rcnn_id, $data, 'plugin_realms', $remote_poller_id);
+	}
+
 	// Auth tables
 	if ($class == 'all' || $class == 'auth') {
 		$data = db_fetch_assoc('SELECT * FROM user_auth');
@@ -1429,7 +1544,9 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 		replicate_out_table($rcnn_id, $data, 'data_input_data', $remote_poller_id);
 	}
 
-	api_plugin_hook_function('replicate_out', $remote_poller_id);
+	if ($class == 'all') {
+		api_plugin_hook_function('replicate_out', array('remote_poller_id' => $remote_poller_id, 'rcnn_id' => $rcnn_id, 'class' => $class));
+	}
 
 	$stats = db_fetch_row_prepared('SELECT
 		SUM(CASE WHEN action=0 THEN 1 ELSE 0 END) AS snmp,
@@ -1446,61 +1563,36 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			array($stats['snmp'], $stats['script'], $stats['server'], $remote_poller_id));
 	}
 
-	raise_message('poller_sync');
+	if ($class != 'plugins' && $config['is_web']) {
+		replicate_log('Synchronization of Poller ' . $remote_poller_id . ' completed', POLLER_VERBOSITY_LOW);
+		raise_message('poller_sync');
+	}
 
 	return true;
 }
 
-function replicate_in() {
-	$replicate_inout_tables = array(
-		'host' => array(
-			'direction'   => 'inout', // Relatively small table
-			'in_freq'     => 'onchange',
-			'out_freq'    => 'onrecovery',
-			'out_columns' => 'status, status_event_count, status_fail_date, status_rec_date, status_last_errors, min_time, max_time, cur_time, avg_time, polling_time, total_polls, failed_polls, availability',
-			'in_columns'  => 'all',
-			'setting'     => 'poller_replicate_device_cache_crc_|poller_id|'
-		),
-		'host_snmp_cache' => array(
-			'direction'   => 'inout', // Potentially large table
-			'in_freq'     => 'onchange',
-			'out_freq'    => 'onrecovery',
-			'out_columns' => 'all_changed',
-			'in_columns'  => 'all',
-			'setting'     => 'poller_replicate_device_cache_crc_|poller_id|'
-		),
-		'poller_reindex' => array(
-			'direction'   => 'inout', // Small table
-			'in_freq'     => 'always',
-			'out_freq'    => 'onrecovery',
-			'out_columns' => 'assert_value',
-			'in_columns'  => 'all',
-			'setting'     => 'poller_replicate_device_cache_crc_|poller_id|'
-		)
-	);
-
-	api_plugin_hook_function('replicate_in');
-}
-
-/** replicate_out_table - replicate out an entire table to
- *  remote database.  Optionally, performs update rather
- *  than a truncate and excluding columns from the on duplicate
- *  clause. By default, the remote table will be recreated
- *  if it's table structure does not match the main table.
+/**
+ * replicate_out_table - replicate out an entire table to
+ *   remote database.  Optionally, performs update rather
+ *   than a truncate and excluding columns from the on duplicate
+ *   clause. By default, the remote table will be recreated
+ *   if it's table structure does not match the main table.
  *
- * @param object $conn          - Connection to remote database
- * @param array $data           - Associative array of the table data
- * @param string $table         - The remote table to replicate to
- * @param int $remote_poller_id - The remote data collector's id
- * @param boolean $truncate     - A flag that if true, truncates, otherwise updates
- * @param array $exclude        - An array of column names to not update on replication
+ * @param  (object) $conn             - Connection to remote database
+ * @param  (array)  $data             - Associative array of the table data
+ * @param  (string) $table            - The remote table to replicate to
+ * @param  (int)    $remote_poller_id - The remote data collector's id
+ * @param  (bool)   $truncate         - A flag that if true, truncates, otherwise updates
+ * @param  (array)  $exclude          - An array of column names to not update on replication
+ *
+ * @return (void)
  */
-function replicate_out_table($conn, &$data, $table, $remote_poller_id, $truncate = true, $exclude = false) {
+function replicate_out_table($conn, &$data, $table, $remote_poller_id, $truncate = true, $exclude = false, $level = POLLER_VERBOSITY_NONE) {
 	if (cacti_sizeof($data)) {
 		/* check if the table structure changed, and if so, recreate */
 		$local_columns  = db_fetch_assoc('SHOW COLUMNS FROM ' . $table);
-		$remote_columns = db_fetch_assoc('SHOW COLUMNS FROM ' . $table, true, $conn);
-		$remote_rows    = db_fetch_cell('SELECT COUNT(*) FROM ' . $table, '', true, $conn);
+		$remote_columns = db_fetch_assoc('SHOW COLUMNS FROM ' . $table, false, $conn);
+		$remote_rows    = db_fetch_cell('SELECT COUNT(*) FROM ' . $table, '', false, $conn);
 
 		if ($exclude !== false && !is_array($exclude)) {
 			$exclude = array($exclude);
@@ -1511,12 +1603,17 @@ function replicate_out_table($conn, &$data, $table, $remote_poller_id, $truncate
 		}
 
 		if (cacti_sizeof($local_columns) != cacti_sizeof($remote_columns)) {
-			cacti_log('NOTE: Replicate Out Detected a Table Structure Change for ' . $table, false, 'REPLICATE');
+			replicate_log('NOTE: Replicate Out Detected a Table Structure Change for ' . $table, $level);
 			$create = db_fetch_row('SHOW CREATE TABLE ' . $table);
-			if (isset($create['Create Table'])) {
-				cacti_log('NOTE: Replication Recreating Remote Table Structure for ' . $table, false, 'REPLICATE');
+			if (isset($create["CREATE TABLE `$table`"]) || isset($create['Create Table'])) {
+				replicate_log('NOTE: Replication Recreating Remote Table Structure for ' . $table, $level);
 				db_execute('DROP TABLE IF EXISTS ' . $table, true, $conn);
-				db_execute($create['Create Table'], true, $conn);
+
+				if (isset($create["CREATE TABLE `$table`"])) {
+					db_execute($create["CREATE TABLE `$table`"], true, $conn);
+				} else {
+					db_execute($create['Create Table'], true, $conn);
+				}
 			}
 		}
 
@@ -1579,7 +1676,13 @@ function replicate_out_table($conn, &$data, $table, $remote_poller_id, $truncate
 
 			if ($rowcnt > 1000) {
 				db_execute($prefix . $sql . $suffix, true, $conn);
-				$rows_done += db_affected_rows($conn);
+				$rows_affected = db_affected_rows($conn);
+				$rows_log      = ((($rows_done % 100000) + $rows_affected) > 100000);
+				$rows_done    += $rows_affected;
+
+				if ($rows_log) {
+					replicate_log('NOTE: Table ' . $table . ' Replicated to Remote Poller ' . $remote_poller_id . ' With ' . $rows_done . ' Rows Updated', $level);
+				}
 				$sql = '';
 				$rowcnt = 0;
 			}
@@ -1590,11 +1693,21 @@ function replicate_out_table($conn, &$data, $table, $remote_poller_id, $truncate
 			$rows_done += db_affected_rows($conn);
 		}
 
-		cacti_log('NOTE: Table ' . $table . ' Replicated to Remote Poller ' . $remote_poller_id . ' With ' . $rows_done . ' Rows Updated', false, 'REPLICATE');
+		replicate_log('INFO: Table ' . $table . ' Replicated to Remote Poller ' . $remote_poller_id . ' With ' . $rows_done . ' Rows Updated', $level);
 	} else {
-		db_execute("TRUNCATE TABLE $table", true, $conn);
+		if (db_table_exists($table, true, $conn)) {
+			db_execute("TRUNCATE TABLE $table", true, $conn);
+		}
 
-		cacti_log('NOTE: Table ' . $table . ' Not Replicated to Remote Poller ' . $remote_poller_id . ' Due to No Rows Found', false, 'REPLICATE');
+		replicate_log('INFO: Table ' . $table . ' Not Replicated to Remote Poller ' . $remote_poller_id . ' Due to No Rows Found', $level);
+	}
+}
+
+function replicate_log($text, $level = POLLER_VERBOSITY_NONE) {
+	if (defined('IN_CACTI_INSTALL') && !defined('IN_PLUGIN_INSTALL')) {
+		log_install_and_file($level, $text, 'REPLICATE', true);
+	} elseif ($level != POLLER_VERBOSITY_NONE) {
+		cacti_log($text, false, 'REPLICATE', $level);
 	}
 }
 
@@ -1701,6 +1814,12 @@ function poller_push_reindex_data_to_poller($device_id = 0, $data_query_id = 0,
 function replicate_table_to_poller($conn, &$data, $table, $exclude = false) {
 	$max_packet  = db_fetch_row("SHOW GLOBAL VARIABLES LIKE 'max_allowed_packet'", true, $conn);
 
+	if (cacti_sizeof($max_packet)) {
+		$max_packet = $max_packet['Value'];
+	} else {
+		$max_packet = 2500000;
+	}
+
 	if (cacti_sizeof($data)) {
 		$prefix    = "INSERT INTO $table (";
 		$suffix    = ' ON DUPLICATE KEY UPDATE ';
@@ -1875,6 +1994,35 @@ function poller_push_table($db_cnn, $records, $table, $ignore = false, $dupes =
 	return cacti_sizeof($records);
 }
 
+/**
+ * remote_poller_up - Given a remote poller id, check if it has responded
+ *   recently, and if so, return true else return false.
+ *
+ * @param   (int) The remote poller id
+ *
+ * @returns (bool) True if up else false
+ */
+function remote_poller_up($poller_id) {
+	$gone_time = read_config_option('poller_interval') * 2;
+
+	if (empty($poller_id) || $poller_id <= 1) {
+		return false;
+	}
+
+	$up_poller = db_fetch_cell_prepared('SELECT COUNT(*)
+		FROM poller
+		WHERE UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) < ?
+		AND id = ?
+		AND disabled = ""',
+		array($gone_time, $poller_id));
+
+	if ($up_poller) {
+		return true;
+	} else {
+		return false;
+	}
+}
+
 function should_ignore_from_replication($path) {
 	$entry = basename($path);
 	return ($entry == '.' || $entry == '..' || $entry == '.git' || $entry == '');
@@ -1938,15 +2086,17 @@ function get_remote_poller_ids_from_devices(&$devices) {
 	}
 }
 
-/** register_process_start - public function to register a process
- *  in Cacti's process table
+/**
+ * register_process_start - public function to register a process
+ *   in Cacti's process table
  *
- * @param string $tasktype  - Mandatory task type
- * @param string $taskname  - Mandatory task name
- * @param int $taskid       - Optional task id
- * @param int $timeout      - Optional timeout
- * @return boolean success  - true if you can start running, else false if
- *                            another version is running and has not ended.
+ * @param  (string) $tasktype - Mandatory task type
+ * @param  (string) $taskname - Mandatory task name
+ * @param  (int)    $taskid   - Optional task id
+ * @param  (int)    $timeout  - Optional timeout
+ *
+ * @return (bool)   success   - true if you can start running, else false if
+ *                              another version is running and has not ended.
  */
 function register_process_start($tasktype, $taskname, $taskid = 0, $timeout = 300) {
 	$pid = getmypid();
@@ -1997,14 +2147,16 @@ function register_process_start($tasktype, $taskname, $taskid = 0, $timeout = 30
 	return true;
 }
 
-/** register_process - register a process in Cacti's process table
+/**
+ * register_process - register a process in Cacti's process table
  *
- * @param string $tasktype  - Mandatory task type
- * @param string $taskname  - Mandatory task name
- * @param int $taskid       - Mandatory task id
- * @param int $pid          - Mandatory pid
- * @param int $timeout      - Mandatory timeout
- * @return null             - No data is returned
+ * @param  (string) $tasktype  - Mandatory task type
+ * @param  (string) $taskname  - Mandatory task name
+ * @param  (int)    $taskid    - Mandatory task id
+ * @param  (int)    $pid       - Mandatory pid
+ * @param  (int)    $timeout   - Mandatory timeout
+ *
+ * @return (void)
  */
 function register_process($tasktype, $taskname, $taskid, $pid, $timeout) {
 	if (!db_table_exists('processes')) {
@@ -2016,13 +2168,15 @@ function register_process($tasktype, $taskname, $taskid, $pid, $timeout) {
 		array($tasktype, $taskname, $taskid, $pid, $timeout));
 }
 
-/** unregister_process - remove a process from Cacti's process table
+/**
+ * unregister_process - remove a process from Cacti's process table
  *
- * @param string $tasktype  - Mandatory task type
- * @param string $taskname  - Mandatory task name
- * @param int $taskid       - Optional task id
- * @param int $pid          - Optional task pid
- * @return null             - No data is returned
+ * @param  (string) $tasktype  - Mandatory task type
+ * @param  (string) $taskname  - Mandatory task name
+ * @param  (int)    $taskid       - Optional task id
+ * @param  (int)    $pid          - Optional task pid
+ *
+ * @return (void)
  */
 function unregister_process($tasktype, $taskname, $taskid = 0, $pid = -1) {
 	if (!db_table_exists('processes')) {
@@ -2045,12 +2199,14 @@ function unregister_process($tasktype, $taskname, $taskid = 0, $pid = -1) {
 	}
 }
 
-/** heartbeat_process - update the process table last_update timestamp
+/**
+ * heartbeat_process - update the process table last_update timestamp
  *
- * @param string $tasktype  - Mandatory task type
- * @param string $taskname  - Mandatory task name
- * @param int $taskid       - Optional task id
- * @return null             - No data is returned
+ * @param  (string) $tasktype  - Mandatory task type
+ * @param  (string) $taskname  - Mandatory task name
+ * @param  (int)    $taskid       - Optional task id
+ *
+ * @return (void)
  */
 function heartbeat_process($tasktype, $taskname, $taskid = 0) {
 	if (!db_table_exists('processes')) {
@@ -2065,14 +2221,16 @@ function heartbeat_process($tasktype, $taskname, $taskid = 0) {
 		array($tasktype, $taskname, $taskid));
 }
 
-/** timeout_kill_registered_processes - allow a Cacti plugin or scheduled task to
- *  be bulk cleaned.
+/**
+ * timeout_kill_registered_processes - allow a Cacti plugin or scheduled task to
+ *   be bulk cleaned.
  *
- * @param string $tasktype  - Optional task type
- * @param string $taskname  - Optional task name
- * @param string $taskid    - Optional task id
- * @param string $pid       - Optional pid
- * @return null             - No data is returned
+ * @param  (string) $tasktype  - Optional task type
+ * @param  (string) $taskname  - Optional task name
+ * @param  (string) $taskid    - Optional task id
+ * @param  (string) $pid       - Optional pid
+ *
+ * @return (void)
  */
 function timeout_kill_registered_processes($tasktype = '', $taskname = '', $taskid = 0, $pid = -1) {
 	if (!db_table_exists('processes')) {
