@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -509,24 +509,36 @@ function push_out_data_input_method($data_input_id) {
 function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items, $poller_id = 1) {
 	global $config;
 
+	$ids    = '';
+	$raised = false;
+
 	/* set all fields present value to 0, to mark the outliers when we are all done */
-	$ids = '';
 	if (cacti_sizeof($local_data_ids)) {
 		$ids = implode(', ', $local_data_ids);
 
 		if ($ids != '') {
 			db_execute_prepared("UPDATE poller_item
-				SET present=0
+				SET present = 0
 				WHERE poller_id = ?
 				AND local_data_id IN ($ids)",
 				array($poller_id));
 
-			if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-				db_execute_prepared("UPDATE poller_item
-					SET present=0
-					WHERE poller_id = ?
-					AND local_data_id IN ($ids)",
-					array($poller_id), true, $rcnn_id);
+			if ($poller_id > 1) {
+				if (remote_poller_up($poller_id)) {
+					if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
+						db_execute_prepared("UPDATE poller_item
+							SET present = 0
+							WHERE poller_id = ?
+							AND local_data_id IN ($ids)",
+							array($poller_id), true, $rcnn_id);
+					} else {
+						raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+						$raised = true;
+					}
+				} else {
+					raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+					$raised = true;
+				}
 			}
 		}
 	} else {
@@ -579,8 +591,18 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 			if ($overhead + $buf_len > $max_packet - 1024) {
 				db_execute($sql_prefix . $buffer . $sql_suffix);
 
-				if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-					db_execute($sql_prefix . $buffer . $sql_suffix, true, $rcnn_id);
+				if ($poller_id > 1) {
+					if (remote_poller_up($poller_id)) {
+						if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
+							db_execute($sql_prefix . $buffer . $sql_suffix, true, $rcnn_id);
+						} elseif (!$raised) {
+							raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+							$raised = true;
+						}
+					} elseif (!$raised) {
+						raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+						$raised = true;
+					}
 				}
 
 				$buffer    = '';
@@ -595,8 +617,18 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 	if ($buf_count > 0) {
 		db_execute($sql_prefix . $buffer . $sql_suffix);
 
-		if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-			db_execute($sql_prefix . $buffer . $sql_suffix, true, $rcnn_id);
+		if ($poller_id > 1) {
+			if (remote_poller_up($poller_id)) {
+				if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
+					db_execute($sql_prefix . $buffer . $sql_suffix, true, $rcnn_id);
+				} else {
+					raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+					$raised = true;
+				}
+			} elseif (!$raised) {
+				raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+				$raised = true;
+			}
 		}
 	}
 
@@ -608,15 +640,21 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 			AND local_data_id IN ($ids)",
 			array($poller_id));
 
-		if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-			db_execute_prepared("DELETE FROM poller_item
-				WHERE present = 0
-				AND poller_id = ?
-				AND local_data_id IN ($ids)",
-				array($poller_id), true, $rcnn_id);
+		if ($poller_id > 1) {
+			if (remote_poller_up($poller_id)) {
+				if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
+					db_execute_prepared("DELETE FROM poller_item
+						WHERE present = 0
+						AND poller_id = ?
+						AND local_data_id IN ($ids)",
+						array($poller_id), true, $rcnn_id);
+				} elseif (!$raised) {
+					raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+				}
+			} elseif (!$raised) {
+				raise_message('poller_down_' . $poller_id, __('Remote Poller %s is Down, you will need to perform a FullSync once it is up again', $poller_id), MESSAGE_LEVEL_WARN);
+			}
 		}
-	} else {
-		/* only handle explicitely given local_data_ids */
 	}
 }
 
@@ -653,12 +691,12 @@ function push_out_host($host_id, $local_data_id = 0, $data_template_id = 0) {
 
 	/* sql WHERE for local_data_id */
 	if ($local_data_id != 0) {
-		$sql_where .= ' AND dl.id=' . $local_data_id;
+		$sql_where .= ' AND dl.id = ' . $local_data_id;
 	}
 
 	/* sql WHERE for data_template_id */
 	if ($data_template_id != 0) {
-		$sql_where .= ' AND dtd.data_template_id=' . $data_template_id;
+		$sql_where .= ' AND dtd.data_template_id = ' . $data_template_id;
 	}
 
 	$data_sources = db_fetch_assoc('SELECT ' . SQL_NO_CACHE . " dtd.id,
@@ -667,8 +705,8 @@ function push_out_host($host_id, $local_data_id = 0, $data_template_id = 0) {
 		dl.snmp_query_id, dl.snmp_index
 		FROM data_local AS dl
 		INNER JOIN data_template_data AS dtd
-		ON dl.id=dtd.local_data_id
-		WHERE dtd.data_input_id>0
+		ON dl.id = dtd.local_data_id
+		WHERE dtd.data_input_id > 0
 		AND (dl.snmp_query_id = 0 OR (dl.snmp_query_id > 0 AND dl.snmp_index != ''))
 		$sql_where");
 
@@ -684,41 +722,62 @@ function push_out_host($host_id, $local_data_id = 0, $data_template_id = 0) {
 			}
 			$host = $hosts[$data_source['host_id']];
 
-			/* get field information FROM the data template */
+			/**
+			 * get field information FROM the data template
+			 */
 			if (!isset($template_fields[$data_source['local_data_template_data_id']])) {
 				$template_fields[$data_source['local_data_template_data_id']] = db_fetch_assoc_prepared('SELECT ' . SQL_NO_CACHE . '
 					did.value, did.t_value, dif.id, dif.type_code
 					FROM data_input_fields AS dif
 					LEFT JOIN data_input_data AS did
-					ON dif.id=did.data_input_field_id
+					ON dif.id = did.data_input_field_id
 					WHERE dif.data_input_id = ?
 					AND did.data_template_data_id = ?
-					AND (did.t_value="" OR did.t_value is null)
+					AND ((did.t_value = "" OR did.t_value IS NULL) OR (did.t_value = "on" AND did.value = ""))
 					AND dif.input_output = "in"',
 					array($data_source['data_input_id'], $data_source['local_data_template_data_id']));
 			}
 
-			/* loop through each field contained in the data template and push out a host value if:
-			 - the field is a valid "host field"
-			 - the value of the field is empty
-			 - the field is set to 'templated' */
+			/**
+			 * loop through each field contained in the data template and push out a host value if:
+			 * - the field is a valid "host field" and it's not set to be over-written
+			 * - in other words the t_value is not checked
+			 */
 			if (cacti_sizeof($template_fields[$data_source['local_data_template_data_id']])) {
 				foreach ($template_fields[$data_source['local_data_template_data_id']] as $template_field) {
-					if (preg_match('/^' . VALID_HOST_FIELDS . '$/i', $template_field['type_code']) && $template_field['value'] == '' && $template_field['t_value'] == '') {
-						// handle special case type_code
-						if ($template_field['type_code'] == 'host_id') {
-							$template_field['type_code'] = 'id';
-						}
+					$update = false;
+					$field  = '';
 
-						db_execute_prepared('REPLACE INTO data_input_data
-							(data_input_field_id, data_template_data_id, value)
-							VALUES (?, ?, ?)',
-							array($template_field['id'], $data_source['id'], $host[$template_field['type_code']]));
+					// handle special case type_code
+					if ($template_field['type_code'] == 'host_id') {
+						$field = 'id';
+					}
+
+					if (preg_match('/^' . VALID_HOST_FIELDS . '$/i', $template_field['type_code']) && $template_field['t_value'] != 'on') {
+						// It's a valid host type-code
+						$update = true;
+
+						if ($template_field['type_code'] != 'host_id') {
+							$field = $template_field['type_code'];
+						}
+					}
+
+					if ($update) {
+						// Don't mess with a field that does not exist
+						// In the host table
+						if (isset($host[$field])) {
+							db_execute_prepared('REPLACE INTO data_input_data
+								(data_input_field_id, data_template_data_id, value)
+								VALUES (?, ?, ?)',
+								array($template_field['id'], $data_source['id'], $host[$field]));
+						}
 					}
 				}
 			}
 
-			/* flag an update to the poller cache as well */
+			/**
+			 * flag an update to the poller cache as well
+			 */
 			$local_data_ids[] = $data_source['local_data_id'];
 
 			/* create a new compatible structure */
