@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2022 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -261,7 +261,18 @@ if (cacti_sizeof($ds_needing_fixes)) {
 	}
 }
 
-/* determine the number of active profiles to improve poller performance
+/**
+ * shortcut some expensive logic in spine if possible the
+ * order by snmp port is a rare thing to be needed and only
+ * needs to occur if your are talking to more than one snmp
+ * agent on a host.
+ */
+$total_ports = db_fetch_cell('SELECT COUNT(DISTINCT snmp_port)
+	FROM poller_item');
+set_config_option('total_snmp_ports', $total_ports);
+
+/**
+ * determine the number of active profiles to improve poller performance
  * under some circumstances.  Save this data for spine and cmd.php.
  */
 $active_profiles = db_fetch_cell('SELECT COUNT(DISTINCT data_source_profile_id)
@@ -484,54 +495,37 @@ while ($poller_runs_completed < $poller_runs) {
 	/* only report issues for the main poller or from bad local
 	 * data ids, other pollers may insert somewhat asynchornously
 	 */
+	$issues = [];
 	$issues_limit = 20;
+	$issues_check = ( $poller_id == 1 || $config['connection'] == 'online');
+	$issues_param = [ $current_time, $poller_id, $poller_id ];
 
-	if ($poller_id == 1) {
-		$issues = db_fetch_assoc_prepared('SELECT ' . SQL_NO_CACHE . ' local_data_id, rrd_name
+	$issues_sql = '
 			FROM poller_output AS po
 			LEFT JOIN data_local AS dl
 			ON po.local_data_id = dl.id
 			LEFT JOIN host AS h
 			ON dl.host_id = h.id
 			WHERE time < FROM_UNIXTIME(? - 600)
+			AND (h.poller_id = ? OR h.id IS NULL or ? = 1)';
+
+	if ($issues_check) {
+		$issues = db_fetch_assoc_prepared('SELECT ' . SQL_NO_CACHE . '
+			local_data_id, rrd_name' . $issues_sql . '
 			LIMIT ' . $issues_limit,
-			array($current_time));
-	} elseif ($config['connection'] == 'online') {
-		$issues = db_fetch_assoc_prepared('SELECT ' . SQL_NO_CACHE . ' local_data_id, rrd_name
-			FROM poller_output AS po
-			LEFT JOIN data_local AS dl
-			ON po.local_data_id = dl.id
-			LEFT JOIN host AS h
-			ON dl.host_id = h.id
-			WHERE h.poller_id = ?
-			OR h.id IS NULL
-			AND time < FROM_UNIXTIME(? - 600)
-			LIMIT ' . $issues_limit,
-			array($poller_id, $current_time));
-	} else{
-		$issues = array();
+			$issues_param);
 	}
 
 	if (cacti_sizeof($issues)) {
-		$count  = db_fetch_cell_prepared('SELECT ' . SQL_NO_CACHE . ' COUNT(*)
-			FROM poller_output AS po
-			LEFT JOIN data_local AS dl
-			ON po.local_data_id = dl.id
-			LEFT JOIN host AS h
-			ON dl.host_id = h.id
-			WHERE (h.poller_id = ? OR h.id IS NULL)
-			AND time < FROM_UNIXTIME(? - 600)',
-			array($poller_id, $current_time));
+		$count  = db_fetch_cell_prepared('SELECT ' . SQL_NO_CACHE . '
+			COUNT(*)' . $issues_sql,
+			$issues_param);
 
-		if (cacti_sizeof($issues)) {
-			$issue_list =  'DS[';
-			$i = 0;
-			foreach($issues as $issue) {
-				$issue_list .= ($i > 0 ? ', ' : '') . $issue['local_data_id'];
-				$i++;
-			}
-			$issue_list .= ']';
+		$issues_list = [];
+		foreach($issues as $issue) {
+			$issues_list []= $issue['local_data_id'];
 		}
+		$issue_list = 'DS[' . implode(', ', $issues_list) . ']';
 
 		if ($count > $issues_limit) {
 			$issue_list .= ", Additional Issues Remain.  Only showing first $issues_limit";
@@ -541,17 +535,10 @@ while ($poller_runs_completed < $poller_runs) {
 
 		admin_email(__('Cacti System Warning'), __('WARNING: Poller Output Table not empty for poller id %d.  Issues: %d, %s.', $poller_id, $count, $issue_list));
 
-		db_execute_prepared('DELETE po
-			FROM poller_output AS po
-			LEFT JOIN data_local AS dl
-			ON po.local_data_id = dl.id
-			LEFT JOIN host AS h
-			ON dl.host_id = h.id
-			WHERE (h.poller_id = ? OR h.id IS NULL)
-			AND time < FROM_UNIXTIME(? - 600)',
-			array($poller_id, $current_time));
+		db_execute_prepared('DELETE po ' . $issues_sql, $issues_param);
 	}
 
+
 	// mainline
 	if (read_config_option('poller_enabled') == 'on') {
 		// determine the number of hosts to process per file
@@ -777,9 +764,10 @@ while ($poller_runs_completed < $poller_runs) {
 				dsstats_poller_bottom();
 				dsdebug_poller_bottom();
 				boost_poller_bottom();
-				api_plugin_hook('poller_bottom');
 			}
 
+			api_plugin_hook('poller_bottom');
+
 			// record the start time for this loop
 			$loop_end      = microtime(true);
 			$cur_loop_time = $loop_end - $loop_start;
@@ -799,7 +787,7 @@ while ($poller_runs_completed < $poller_runs) {
 
 			$plugin_end = microtime(true);
 			if ($sleep_time > 0) {
-				usleep($sleep_time * 1000000);
+				usleep(intval($sleep_time * 1000000));
 			}
 
 			api_plugin_hook('poller_top');
@@ -838,6 +826,7 @@ if ($poller_id == 1) {
 
 	automation_poller_bottom();
 	poller_maintenance();
+	api_plugin_hook('poller_bottom');
 }
 
 function bad_index_check($mibs) {
@@ -938,18 +927,23 @@ function poller_enabled_check($poller_id) {
 
 	$system_enabled = read_config_option('poller_enabled');
 
+	$should_exit = false;
 	if ($system_enabled == '') {
 		cacti_log('WARNING: System Polling is Disabled!  Therefore, data collection from the poller will be suspended till re-enabled.', true, 'SYSTEM');
+		$should_exit = true;
+	}
 
-		exit(1);
-	} elseif ($poller_disabled == 'on') {
+	if ($poller_disabled == 'on') {
+		cacti_log('WARNING: Poller ' . $poller_id . ' is Disabled.  Therefore, data collection for this Poller will be suspended till it\'s re-enabled.', true, 'SYSTEM');
+		$should_exit = true;
+	}
+
+	if ($should_exit) {
 		db_execute_prepared('UPDATE poller
 			SET last_status=NOW()
 			WHERE id = ?',
 			array($poller_id), true, $poller_db_cnn_id);
 
-		cacti_log('WARNING: Poller ' . $poller_id . ' is Disabled.  Therefore, data collection for this Poller will be suspended till it\'s re-enabled.', true, 'SYSTEM');
-
 		exit(1);
 	}
 }
@@ -1071,4 +1065,3 @@ function display_help() {
 	print "    --force        Override poller overrun detection and force a poller run.\n";
 	print "    --debug|-d     Output debug information.  Similar to cacti's DEBUG logging level.\n\n";
 }
-
